#!/usr/bin/env node

'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var program = _interopDefault(require('commander'));
var chalk = _interopDefault(require('chalk'));
var fileSize = _interopDefault(require('filesize'));
var path = require('path');
var prettyMs = _interopDefault(require('pretty-ms'));
var readPkg = _interopDefault(require('read-pkg'));
var slash = _interopDefault(require('slash'));
var __ = require('..');
var fs = require('fs');
var rollup = require('rollup');

var name = "bundlib";
var displayName = "Bundlib";
var version = "0.15.1";

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function oneByOne(arr, callback) {
  let index = 0;
  const len = arr.length;
  const last = len - 1;

  const next = () => {
    if (index > last) {
      return;
    }

    callback(arr[index], index < last ? next : null);
    index++;
  };

  next();
}

function _catch(body, recover) {
  try {
    var result = body();
  } catch (e) {
    return recover(e);
  }

  if (result && result.then) {
    return result.then(void 0, recover);
  }

  return result;
}

function build(configs, callbacks) {
  if (callbacks.start) {
    callbacks.start();
  }

  const cache = {};
  oneByOne(configs, function (config, next) {
    try {
      let _exit = false;

      function _temp2(_result) {
        if (_exit) return _result;

        if (next) {
          return next();
        }

        if (callbacks.end) {
          callbacks.end();
        }
      }

      const {
        input,
        output
      } = config;
      const {
        file: outputFile,
        format
      } = output;
      const isModuleBuild = format === 'cjs' || format === 'es';
      const cacheKey = `${isModuleBuild ? 'module' : 'browser'}:${input}`;
      config.cache = cache[cacheKey];

      if (callbacks.buildStart) {
        callbacks.buildStart(input, outputFile);
      }

      const _temp = _catch(function () {
        const currentTime = Date.now();
        return Promise.resolve(rollup.rollup(config)).then(function (buildResult) {
          config.cache = cache[cacheKey] = buildResult.cache;
          return Promise.resolve(buildResult.write(output)).then(function () {
            const {
              size
            } = fs.statSync(outputFile);
            const totalTime = Date.now() - currentTime;

            if (callbacks.buildEnd) {
              callbacks.buildEnd(outputFile, size, totalTime);
            }
          });
        });
      }, function (err) {
        if (callbacks.error) {
          callbacks.error(err);
        }

        if (callbacks.end) {
          callbacks.end();
        }

        throw err;
      });

      return Promise.resolve(_temp && _temp.then ? _temp.then(_temp2) : _temp2(_temp));
    } catch (e) {
      return Promise.reject(e);
    }
  });
}

function watch(configs, callbacks) {
  const {
    start,
    end,
    buildStart,
    buildEnd,
    error
  } = callbacks;

  const ERROR = error && (event => {
    error(event.error);
  });

  const table = {
    START: start && (() => {
      start();
    }),
    END: end && (() => {
      end();
    }),
    BUNDLE_START: buildStart && (event => {
      const {
        output: out
      } = event;
      const {
        length: len
      } = out;

      for (let i = 0; i < len; i++) {
        buildStart(event.input, out[i]);
      }
    }),
    BUNDLE_END: buildEnd && (event => {
      const {
        output: out
      } = event;
      const {
        length: len
      } = out;

      for (let i = 0; i < len; i++) {
        const stats = fs.statSync(out[i]);
        buildEnd(out[i], stats.size, event.duration || 0);
      }
    }),
    ERROR,
    FATAL: ERROR
  };
  const watcher = rollup.watch(configs);
  watcher.on('event', event => {
    const {
      code
    } = event;
    const handle = table[code];

    if (handle) {
      handle(event);
    }
  });
}

const bundlib = function (cwd, options, callbacks, pkg) {
  try {
    return Promise.resolve(__.configsFromPkg(cwd, options, pkg)).then(function (configs) {
      const {
        warn
      } = callbacks;
      const onwarn = warn ? warning => warn(warning) : () => null;
      configs.forEach(config => {
        config.onwarn = onwarn;
      });
      (options.watch ? watch : build)(configs, callbacks);
    });
  } catch (e) {
    return Promise.reject(e);
  }
};

function create(name) {
  const method = console[name];
  return msg => method(chalk.cyan(msg));
}

const log = create('log');
const error = create('error');
function logError(err) {
  if (err.stack) {
    error(err.stack);
  }

  const tag = chalk.bold.bgWhite.red.inverse(' error ');
  error(`${tag} ${err.message || err}`);
}

function _catch$1(body, recover) {
  try {
    var result = body();
  } catch (e) {
    return recover(e);
  }

  if (result && result.then) {
    return result.then(void 0, recover);
  }

  return result;
}

const action = function (displayName, version, silent, options) {
  try {
    function _temp3() {
      let buildIndex = 0;
      const callbacks = {
        error(err) {
          logError(err);
          process.exit(1);
        }

      };

      if (!silent) {
        _extends(callbacks, {
          buildEnd(filename, size, duration) {
            const tag = green.bgBlack.inverse(' built ');
            const path$1 = yellow(slash(path.relative(cwd, filename)));
            const colorSize = magenta(fileSize(size));
            const colorTime = magenta(prettyMs(duration, {
              secondsDecimalDigits: 2
            }));
            log(`${tag} ${path$1} ( ${colorSize} in ${colorTime} )`);
          },

          error(err) {
            logError(err);
          },

          warn(warning) {
            let message = warning;

            if (typeof message === 'object') {
              const {
                plugin,
                message: msg
              } = message;
              message = msg;

              if (plugin) {
                message = `(plugin ${magenta(plugin)}) ${message}`;
              }
            }

            const tag = magenta('warning:');
            log(`${tag} ${message}`);
          }

        });

        if (options.watch) {
          _extends(callbacks, {
            start() {
              if (buildIndex) {
                log(`rebuilding...
`);
              }

              buildIndex++;
            },

            end() {
              log(`
waiting for changes...`);
            }

          });
        }
      }

      const _temp = _catch$1(function () {
        return Promise.resolve(bundlib(cwd, options, callbacks, pkg)).then(function () {});
      }, function (err) {
        logError(err);
        process.exit(1);
      });

      if (_temp && _temp.then) return _temp.then(function () {});
    }

    const cwd = process.cwd();
    let pkg;

    const _temp2 = function () {
      if (!silent) {
        const appInfo = prjInfo(displayName, version);
        log(`${appInfo}
`);
        const filename = yellow('package.json');
        log(`reading: ${filename}`);
        return Promise.resolve(readPkg({
          cwd,
          normalize: false
        })).then(function (_readPkg) {
          pkg = _readPkg;
          const {
            name: prjName,
            displayName: prjDispName,
            version: prjVer
          } = pkg;
          const prjInfoName = prjDispName || prjName;

          if (prjInfoName && prjVer) {
            const info = prjInfo(prjInfoName, prjVer);
            log(`building: ${info}
`);
          }
        });
      }
    }();

    return Promise.resolve(_temp2 && _temp2.then ? _temp2.then(_temp3) : _temp3(_temp2));
  } catch (e) {
    return Promise.reject(e);
  }
};
const {
  bold
} = chalk;
const green = bold.green;
const yellow = bold.yellow;
const magenta = bold.magenta;

function prjInfo(name, ver) {
  const projName = green(name);
  const projVer = yellow(`v${ver}`);
  return `${projName} ${projVer}`;
}

if (!chalk.level && 'MINGW_CHOST' in process.env) {
  chalk.level = 1;
}

program.version(version, '-v, --version').name(name).option('-d, --dev', 'create development builds').option('-w, --watch', 'run bundlib in watch mode').option('-s, --silent', 'prevent messages from showing in the console').action(function () {
  try {
    const {
      dev,
      watch,
      silent
    } = program;
    return Promise.resolve(action(displayName, version, silent, {
      dev,
      watch
    })).then(function () {});
  } catch (e) {
    return Promise.reject(e);
  }
}).parseAsync(process.argv);
//# sourceMappingURL=bundlib-cli.js.map
